/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

var util = require('util');

/**
 * @class
 * Initializes a new instance of the USqlTableStatistics class.
 * @constructor
 * A Data Lake Analytics catalog U-SQL table statistics item.
 *
 * @member {string} [databaseName] the name of the database.
 * 
 * @member {string} [schemaName] the name of the schema associated with this
 * table and database.
 * 
 * @member {string} [tableName] the name of the table.
 * 
 * @member {string} [name] the name of the table statistics.
 * 
 * @member {string} [userStatName] the name of the user statistics.
 * 
 * @member {string} [statDataPath] the path to the statistics data.
 * 
 * @member {date} [createTime] the creation time of the statistics.
 * 
 * @member {date} [updateTime] the last time the statistics were updated.
 * 
 * @member {boolean} [isUserCreated] the switch indicating if these statistics
 * are user created.
 * 
 * @member {boolean} [isAutoCreated] the switch indicating if these statistics
 * are automatically created.
 * 
 * @member {boolean} [hasFilter] the switch indicating if these statistics
 * have a filter.
 * 
 * @member {string} [filterDefinition] the filter definition for the
 * statistics.
 * 
 * @member {array} [colNames] the list of column names associated with these
 * statistics.
 * 
 */
function USqlTableStatistics() {
  USqlTableStatistics['super_'].call(this);
}

util.inherits(USqlTableStatistics, models['CatalogItem']);

/**
 * Defines the metadata of USqlTableStatistics
 *
 * @returns {object} metadata of USqlTableStatistics
 *
 */
USqlTableStatistics.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'USqlTableStatistics',
    type: {
      name: 'Composite',
      className: 'USqlTableStatistics',
      modelProperties: {
        computeAccountName: {
          required: false,
          serializedName: 'computeAccountName',
          type: {
            name: 'String'
          }
        },
        version: {
          required: false,
          serializedName: 'version',
          type: {
            name: 'String'
          }
        },
        databaseName: {
          required: false,
          serializedName: 'databaseName',
          type: {
            name: 'String'
          }
        },
        schemaName: {
          required: false,
          serializedName: 'schemaName',
          type: {
            name: 'String'
          }
        },
        tableName: {
          required: false,
          serializedName: 'tableName',
          type: {
            name: 'String'
          }
        },
        name: {
          required: false,
          serializedName: 'statisticsName',
          type: {
            name: 'String'
          }
        },
        userStatName: {
          required: false,
          serializedName: 'userStatName',
          type: {
            name: 'String'
          }
        },
        statDataPath: {
          required: false,
          serializedName: 'statDataPath',
          type: {
            name: 'String'
          }
        },
        createTime: {
          required: false,
          serializedName: 'createTime',
          type: {
            name: 'DateTime'
          }
        },
        updateTime: {
          required: false,
          serializedName: 'updateTime',
          type: {
            name: 'DateTime'
          }
        },
        isUserCreated: {
          required: false,
          serializedName: 'isUserCreated',
          type: {
            name: 'Boolean'
          }
        },
        isAutoCreated: {
          required: false,
          serializedName: 'isAutoCreated',
          type: {
            name: 'Boolean'
          }
        },
        hasFilter: {
          required: false,
          serializedName: 'hasFilter',
          type: {
            name: 'Boolean'
          }
        },
        filterDefinition: {
          required: false,
          serializedName: 'filterDefinition',
          type: {
            name: 'String'
          }
        },
        colNames: {
          required: false,
          serializedName: 'colNames',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        }
      }
    }
  };
};

module.exports = USqlTableStatistics;
