/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * Catalog
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the DataLakeAnalyticsCatalogManagementClient.
 */
export interface Catalog {

    /**
     * Creates the specified secret for use with external data sources in the
     * specified database.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database in which to create
     * the secret.
     * 
     * @param {string} secretName The name of the secret.
     * 
     * @param {object} parameters The parameters required to create the secret
     * (name and password)
     * 
     * @param {string} [parameters.password] the password for the secret to pass in
     * 
     * @param {string} [parameters.uri] the URI identifier for the secret in the
     * format <hostname>:<port>
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    createSecret(accountName: string, databaseName: string, secretName: string, parameters: models.DataLakeAnalyticsCatalogSecretCreateOrUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlSecret>): void;
    createSecret(accountName: string, databaseName: string, secretName: string, parameters: models.DataLakeAnalyticsCatalogSecretCreateOrUpdateParameters, callback: ServiceCallback<models.USqlSecret>): void;

    /**
     * Modifies the specified secret for use with external data sources in the
     * specified database
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the secret.
     * 
     * @param {string} secretName The name of the secret.
     * 
     * @param {object} parameters The parameters required to modify the secret
     * (name and password)
     * 
     * @param {string} [parameters.password] the password for the secret to pass in
     * 
     * @param {string} [parameters.uri] the URI identifier for the secret in the
     * format <hostname>:<port>
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    updateSecret(accountName: string, databaseName: string, secretName: string, parameters: models.DataLakeAnalyticsCatalogSecretCreateOrUpdateParameters, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlSecret>): void;
    updateSecret(accountName: string, databaseName: string, secretName: string, parameters: models.DataLakeAnalyticsCatalogSecretCreateOrUpdateParameters, callback: ServiceCallback<models.USqlSecret>): void;

    /**
     * Gets the specified secret in the specified database
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the secret.
     * 
     * @param {string} secretName The name of the secret to get
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getSecret(accountName: string, databaseName: string, secretName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlSecret>): void;
    getSecret(accountName: string, databaseName: string, secretName: string, callback: ServiceCallback<models.USqlSecret>): void;

    /**
     * Deletes the specified secret in the specified database
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the secret.
     * 
     * @param {string} secretName The name of the secret to delete
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    deleteSecret(accountName: string, databaseName: string, secretName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
    deleteSecret(accountName: string, databaseName: string, secretName: string, callback: ServiceCallback<void>): void;

    /**
     * Deletes all secrets in the specified database
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the secret.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    deleteAllSecrets(accountName: string, databaseName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
    deleteAllSecrets(accountName: string, databaseName: string, callback: ServiceCallback<void>): void;

    /**
     * Retrieves the specified external data source from the Data Lake Analytics
     * catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * external data source.
     * 
     * @param {string} externalDataSourceName The name of the external data source.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getExternalDataSource(accountName: string, databaseName: string, externalDataSourceName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlExternalDataSource>): void;
    getExternalDataSource(accountName: string, databaseName: string, externalDataSourceName: string, callback: ServiceCallback<models.USqlExternalDataSource>): void;

    /**
     * Retrieves the list of external data sources from the Data Lake Analytics
     * catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * external data sources.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listExternalDataSources(accountName: string, databaseName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlExternalDataSourceList>): void;
    listExternalDataSources(accountName: string, databaseName: string, callback: ServiceCallback<models.USqlExternalDataSourceList>): void;

    /**
     * Retrieves the specified credential from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the schema.
     * 
     * @param {string} credentialName The name of the credential.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getCredential(accountName: string, databaseName: string, credentialName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlCredential>): void;
    getCredential(accountName: string, databaseName: string, credentialName: string, callback: ServiceCallback<models.USqlCredential>): void;

    /**
     * Retrieves the list of credentials from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the schema.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listCredentials(accountName: string, databaseName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlCredentialList>): void;
    listCredentials(accountName: string, databaseName: string, callback: ServiceCallback<models.USqlCredentialList>): void;

    /**
     * Retrieves the specified procedure from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * procedure.
     * 
     * @param {string} schemaName The name of the schema containing the procedure.
     * 
     * @param {string} procedureName The name of the procedure.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getProcedure(accountName: string, databaseName: string, schemaName: string, procedureName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlProcedure>): void;
    getProcedure(accountName: string, databaseName: string, schemaName: string, procedureName: string, callback: ServiceCallback<models.USqlProcedure>): void;

    /**
     * Retrieves the list of procedures from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * procedures.
     * 
     * @param {string} schemaName The name of the schema containing the procedures.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listProcedures(accountName: string, databaseName: string, schemaName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlProcedureList>): void;
    listProcedures(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlProcedureList>): void;

    /**
     * Retrieves the specified table from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the table.
     * 
     * @param {string} schemaName The name of the schema containing the table.
     * 
     * @param {string} tableName The name of the table.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getTable(accountName: string, databaseName: string, schemaName: string, tableName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTable>): void;
    getTable(accountName: string, databaseName: string, schemaName: string, tableName: string, callback: ServiceCallback<models.USqlTable>): void;

    /**
     * Retrieves the list of tables from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the tables.
     * 
     * @param {string} schemaName The name of the schema containing the tables.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTables(accountName: string, databaseName: string, schemaName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableList>): void;
    listTables(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlTableList>): void;

    /**
     * Retrieves the specified table type from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the table
     * type.
     * 
     * @param {string} schemaName The name of the schema containing the table type.
     * 
     * @param {string} tableTypeName The name of the table type to retrieve.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getTableType(accountName: string, databaseName: string, schemaName: string, tableTypeName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableType>): void;
    getTableType(accountName: string, databaseName: string, schemaName: string, tableTypeName: string, callback: ServiceCallback<models.USqlTableType>): void;

    /**
     * Retrieves the list of table types from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the table
     * types.
     * 
     * @param {string} schemaName The name of the schema containing the table
     * types.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTableTypes(accountName: string, databaseName: string, schemaName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableTypeList>): void;
    listTableTypes(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlTableTypeList>): void;

    /**
     * Retrieves the specified view from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the view.
     * 
     * @param {string} schemaName The name of the schema containing the view.
     * 
     * @param {string} viewName The name of the view.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getView(accountName: string, databaseName: string, schemaName: string, viewName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlView>): void;
    getView(accountName: string, databaseName: string, schemaName: string, viewName: string, callback: ServiceCallback<models.USqlView>): void;

    /**
     * Retrieves the list of views from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the views.
     * 
     * @param {string} schemaName The name of the schema containing the views.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listViews(accountName: string, databaseName: string, schemaName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlViewList>): void;
    listViews(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlViewList>): void;

    /**
     * Retrieves the specified table statistics from the Data Lake Analytics
     * catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * statistics.
     * 
     * @param {string} schemaName The name of the schema containing the statistics.
     * 
     * @param {string} tableName The name of the table containing the statistics.
     * 
     * @param {string} statisticsName The name of the table statistics.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getTableStatistic(accountName: string, databaseName: string, schemaName: string, tableName: string, statisticsName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableStatistics>): void;
    getTableStatistic(accountName: string, databaseName: string, schemaName: string, tableName: string, statisticsName: string, callback: ServiceCallback<models.USqlTableStatistics>): void;

    /**
     * Retrieves the list of table statistics from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * statistics.
     * 
     * @param {string} schemaName The name of the schema containing the statistics.
     * 
     * @param {string} tableName The name of the table containing the statistics.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTableStatistics(accountName: string, databaseName: string, schemaName: string, tableName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableStatisticsList>): void;
    listTableStatistics(accountName: string, databaseName: string, schemaName: string, tableName: string, callback: ServiceCallback<models.USqlTableStatisticsList>): void;

    /**
     * Retrieves the specified table partition from the Data Lake Analytics
     * catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * partition.
     * 
     * @param {string} schemaName The name of the schema containing the partition.
     * 
     * @param {string} tableName The name of the table containing the partition.
     * 
     * @param {string} partitionName The name of the table partition.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getTablePartition(accountName: string, databaseName: string, schemaName: string, tableName: string, partitionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTablePartition>): void;
    getTablePartition(accountName: string, databaseName: string, schemaName: string, tableName: string, partitionName: string, callback: ServiceCallback<models.USqlTablePartition>): void;

    /**
     * Retrieves the list of table partitions from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * partitions.
     * 
     * @param {string} schemaName The name of the schema containing the partitions.
     * 
     * @param {string} tableName The name of the table containing the partitions.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTablePartitions(accountName: string, databaseName: string, schemaName: string, tableName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTablePartitionList>): void;
    listTablePartitions(accountName: string, databaseName: string, schemaName: string, tableName: string, callback: ServiceCallback<models.USqlTablePartitionList>): void;

    /**
     * Retrieves the list of types within the specified database and schema from
     * the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the types.
     * 
     * @param {string} schemaName The name of the schema containing the types.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTypes(accountName: string, databaseName: string, schemaName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTypeList>): void;
    listTypes(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlTypeList>): void;

    /**
     * Retrieves the specified table valued function from the Data Lake Analytics
     * catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the table
     * valued function.
     * 
     * @param {string} schemaName The name of the schema containing the table
     * valued function.
     * 
     * @param {string} tableValuedFunctionName The name of the tableValuedFunction.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getTableValuedFunction(accountName: string, databaseName: string, schemaName: string, tableValuedFunctionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableValuedFunction>): void;
    getTableValuedFunction(accountName: string, databaseName: string, schemaName: string, tableValuedFunctionName: string, callback: ServiceCallback<models.USqlTableValuedFunction>): void;

    /**
     * Retrieves the list of table valued functions from the Data Lake Analytics
     * catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the table
     * valued functions.
     * 
     * @param {string} schemaName The name of the schema containing the table
     * valued functions.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTableValuedFunctions(accountName: string, databaseName: string, schemaName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableValuedFunctionList>): void;
    listTableValuedFunctions(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlTableValuedFunctionList>): void;

    /**
     * Retrieves the specified assembly from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * assembly.
     * 
     * @param {string} assemblyName The name of the assembly.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getAssembly(accountName: string, databaseName: string, assemblyName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlAssembly>): void;
    getAssembly(accountName: string, databaseName: string, assemblyName: string, callback: ServiceCallback<models.USqlAssembly>): void;

    /**
     * Retrieves the list of assemblies from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the
     * assembly.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listAssemblies(accountName: string, databaseName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlAssemblyList>): void;
    listAssemblies(accountName: string, databaseName: string, callback: ServiceCallback<models.USqlAssemblyList>): void;

    /**
     * Retrieves the specified schema from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the schema.
     * 
     * @param {string} schemaName The name of the schema.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getSchema(accountName: string, databaseName: string, schemaName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlSchema>): void;
    getSchema(accountName: string, databaseName: string, schemaName: string, callback: ServiceCallback<models.USqlSchema>): void;

    /**
     * Retrieves the list of schemas from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database containing the schema.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listSchemas(accountName: string, databaseName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlSchemaList>): void;
    listSchemas(accountName: string, databaseName: string, callback: ServiceCallback<models.USqlSchemaList>): void;

    /**
     * Retrieves the specified database from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {string} databaseName The name of the database.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getDatabase(accountName: string, databaseName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlDatabase>): void;
    getDatabase(accountName: string, databaseName: string, callback: ServiceCallback<models.USqlDatabase>): void;

    /**
     * Retrieves the list of databases from the Data Lake Analytics catalog.
     *
     * @param {string} accountName The Azure Data Lake Analytics account to
     * execute catalog operations on.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {string} [options.filter] OData filter. Optional.
     * 
     * @param {number} [options.top] The number of items to return. Optional.
     * 
     * @param {number} [options.skip] The number of items to skip over before
     * returning elements. Optional.
     * 
     * @param {string} [options.expand] OData expansion. Expand related resources
     * in line with the retrieved resources, e.g. Categories?$expand=Products
     * would expand Product data in line with each Category entry. Optional.
     * 
     * @param {string} [options.select] OData Select statement. Limits the
     * properties on each entry to just those requested, e.g.
     * Categories?$select=CategoryName,Description. Optional.
     * 
     * @param {string} [options.orderby] OrderBy clause. One or more
     * comma-separated expressions with an optional "asc" (the default) or "desc"
     * depending on the order you'd like the values sorted, e.g.
     * Categories?$orderby=CategoryName desc. Optional.
     * 
     * @param {boolean} [options.count] The Boolean value of true or false to
     * request a count of the matching resources included with the resources in
     * the response, e.g. Categories?$count=true. Optional.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listDatabases(accountName: string, options: { filter? : string, top? : number, skip? : number, expand? : string, select? : string, orderby? : string, count? : boolean, customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlDatabaseList>): void;
    listDatabases(accountName: string, callback: ServiceCallback<models.USqlDatabaseList>): void;

    /**
     * Retrieves the list of external data sources from the Data Lake Analytics
     * catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listExternalDataSourcesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlExternalDataSourceList>): void;
    listExternalDataSourcesNext(nextPageLink: string, callback: ServiceCallback<models.USqlExternalDataSourceList>): void;

    /**
     * Retrieves the list of credentials from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listCredentialsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlCredentialList>): void;
    listCredentialsNext(nextPageLink: string, callback: ServiceCallback<models.USqlCredentialList>): void;

    /**
     * Retrieves the list of procedures from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listProceduresNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlProcedureList>): void;
    listProceduresNext(nextPageLink: string, callback: ServiceCallback<models.USqlProcedureList>): void;

    /**
     * Retrieves the list of tables from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTablesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableList>): void;
    listTablesNext(nextPageLink: string, callback: ServiceCallback<models.USqlTableList>): void;

    /**
     * Retrieves the list of table types from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTableTypesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableTypeList>): void;
    listTableTypesNext(nextPageLink: string, callback: ServiceCallback<models.USqlTableTypeList>): void;

    /**
     * Retrieves the list of views from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listViewsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlViewList>): void;
    listViewsNext(nextPageLink: string, callback: ServiceCallback<models.USqlViewList>): void;

    /**
     * Retrieves the list of table statistics from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTableStatisticsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableStatisticsList>): void;
    listTableStatisticsNext(nextPageLink: string, callback: ServiceCallback<models.USqlTableStatisticsList>): void;

    /**
     * Retrieves the list of table partitions from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTablePartitionsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTablePartitionList>): void;
    listTablePartitionsNext(nextPageLink: string, callback: ServiceCallback<models.USqlTablePartitionList>): void;

    /**
     * Retrieves the list of types within the specified database and schema from
     * the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTypesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTypeList>): void;
    listTypesNext(nextPageLink: string, callback: ServiceCallback<models.USqlTypeList>): void;

    /**
     * Retrieves the list of table valued functions from the Data Lake Analytics
     * catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listTableValuedFunctionsNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlTableValuedFunctionList>): void;
    listTableValuedFunctionsNext(nextPageLink: string, callback: ServiceCallback<models.USqlTableValuedFunctionList>): void;

    /**
     * Retrieves the list of assemblies from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listAssembliesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlAssemblyList>): void;
    listAssembliesNext(nextPageLink: string, callback: ServiceCallback<models.USqlAssemblyList>): void;

    /**
     * Retrieves the list of schemas from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listSchemasNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlSchemaList>): void;
    listSchemasNext(nextPageLink: string, callback: ServiceCallback<models.USqlSchemaList>): void;

    /**
     * Retrieves the list of databases from the Data Lake Analytics catalog.
     *
     * @param {string} nextPageLink The NextLink from the previous successful call
     * to List operation.
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listDatabasesNext(nextPageLink: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.USqlDatabaseList>): void;
    listDatabasesNext(nextPageLink: string, callback: ServiceCallback<models.USqlDatabaseList>): void;
}
