/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
*/

import { ServiceClientOptions, RequestOptions, ServiceCallback } from 'ms-rest';
import * as models from '../models';


/**
 * @class
 * WorkspaceCollections
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the PowerBIEmbeddedManagementClient.
 */
export interface WorkspaceCollections {

    /**
     * Retrieves an existing Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getByName(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollection>): void;
    getByName(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<models.WorkspaceCollection>): void;

    /**
     * Creates a new Power BI Workspace Collection with the specified properties.
     * A Power BI Workspace Collection contains one or more Power BI Workspaces
     * and can be used to provision keys that provide API access to those Power
     * BI Workspaces.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} body Create workspace collection request
     * 
     * @param {string} [body.location] Azure location
     * 
     * @param {object} [body.tags]
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    create(resourceGroupName: string, workspaceCollectionName: string, body: models.CreateWorkspaceCollectionRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollection>): void;
    create(resourceGroupName: string, workspaceCollectionName: string, body: models.CreateWorkspaceCollectionRequest, callback: ServiceCallback<models.WorkspaceCollection>): void;

    /**
     * Update an existing Power BI Workspace Collection with the specified
     * properties.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} body Update workspace collection request
     * 
     * @param {object} [body.tags]
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    update(resourceGroupName: string, workspaceCollectionName: string, body: models.UpdateWorkspaceCollectionRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollection>): void;
    update(resourceGroupName: string, workspaceCollectionName: string, body: models.UpdateWorkspaceCollectionRequest, callback: ServiceCallback<models.WorkspaceCollection>): void;

    /**
     * Delete a Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    deleteMethod(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
    deleteMethod(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<void>): void;

    /**
     * Delete a Power BI Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    beginDeleteMethod(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
    beginDeleteMethod(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<void>): void;

    /**
     * Check that the specified Power BI Workspace Collection name is valid and
     * not in use.
     *
     * @param {string} location Azure location
     * 
     * @param {object} body Check name availability request
     * 
     * @param {string} [body.name] Workspace collection name
     * 
     * @param {string} [body.type] Resource type
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    checkNameAvailability(location: string, body: models.CheckNameRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.CheckNameResponse>): void;
    checkNameAvailability(location: string, body: models.CheckNameRequest, callback: ServiceCallback<models.CheckNameResponse>): void;

    /**
     * Retrieves all existing Power BI Workspace Collections in the specified
     * resource group.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listByResourceGroup(resourceGroupName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionList>): void;
    listByResourceGroup(resourceGroupName: string, callback: ServiceCallback<models.WorkspaceCollectionList>): void;

    /**
     * Retrieves all existing Power BI Workspace Collections in the specified
     * subscription.
     *
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    listBySubscription(options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionList>): void;
    listBySubscription(callback: ServiceCallback<models.WorkspaceCollectionList>): void;

    /**
     * Retrieves the primary and secondary access keys for the specified Power BI
     * Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    getAccessKeys(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;
    getAccessKeys(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;

    /**
     * Regenerates the primary or secondary access key for the specified Power BI
     * Workspace Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} body Access key to regenerate
     * 
     * @param {string} [body.keyName] Key name. Possible values include: 'key1',
     * 'key2'
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    regenerateKey(resourceGroupName: string, workspaceCollectionName: string, body: models.WorkspaceCollectionAccessKey, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;
    regenerateKey(resourceGroupName: string, workspaceCollectionName: string, body: models.WorkspaceCollectionAccessKey, callback: ServiceCallback<models.WorkspaceCollectionAccessKeys>): void;

    /**
     * Migrates an existing Power BI Workspace Collection to a different resource
     * group and/or subscription.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {object} body Workspace migration request
     * 
     * @param {string} [body.targetResourceGroup] Name of the resource group that
     * the Power BI Workspace Collections will be migrated to.
     * 
     * @param {array} [body.resources]
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    migrate(resourceGroupName: string, body: models.MigrateWorkspaceCollectionRequest, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<void>): void;
    migrate(resourceGroupName: string, body: models.MigrateWorkspaceCollectionRequest, callback: ServiceCallback<void>): void;
}

/**
 * @class
 * Workspaces
 * __NOTE__: An instance of this class is automatically created for an
 * instance of the PowerBIEmbeddedManagementClient.
 */
export interface Workspaces {

    /**
     * Retrieves all existing Power BI Workspaces in the specified Workspace
     * Collection.
     *
     * @param {string} resourceGroupName Azure resource group
     * 
     * @param {string} workspaceCollectionName Power BI Embedded workspace
     * collection name
     * 
     * @param {object} [options] Optional Parameters.
     * 
     * @param {object} [options.customHeaders] Headers that will be added to the
     * request
     * 
     * @param {ServiceCallback} [callback] callback function; see ServiceCallback
     * doc in ms-rest index.d.ts for details
     */
    list(resourceGroupName: string, workspaceCollectionName: string, options: { customHeaders? : { [headerName: string]: string; } }, callback: ServiceCallback<models.WorkspaceList>): void;
    list(resourceGroupName: string, workspaceCollectionName: string, callback: ServiceCallback<models.WorkspaceList>): void;
}
