/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

/**
 * @class
 * Initializes a new instance of the DeploymentValidateResult class.
 * @constructor
 * Information from validate template deployment response.
 *
 * @member {object} [error] Validation error.
 * 
 * @member {string} [error.code] The error code returned from the server.
 * 
 * @member {string} [error.message] The error message returned from the server.
 * 
 * @member {string} [error.target] The target of the error.
 * 
 * @member {array} [error.details] Validation error.
 * 
 * @member {object} [properties] The template deployment properties.
 * 
 * @member {string} [properties.provisioningState] The state of the
 * provisioning.
 * 
 * @member {string} [properties.correlationId] The correlation ID of the
 * deployment.
 * 
 * @member {date} [properties.timestamp] The timestamp of the template
 * deployment.
 * 
 * @member {object} [properties.outputs] Key/value pairs that represent
 * deploymentoutput.
 * 
 * @member {array} [properties.providers] The list of resource providers
 * needed for the deployment.
 * 
 * @member {array} [properties.dependencies] The list of deployment
 * dependencies.
 * 
 * @member {object} [properties.template] The template content. Use only one
 * of Template or TemplateLink.
 * 
 * @member {object} [properties.templateLink] The URI referencing the
 * template. Use only one of Template or TemplateLink.
 * 
 * @member {string} [properties.templateLink.uri] URI referencing the template.
 * 
 * @member {string} [properties.templateLink.contentVersion] If included it
 * must match the ContentVersion in the template.
 * 
 * @member {object} [properties.parameters] Deployment parameters. Use only
 * one of Parameters or ParametersLink.
 * 
 * @member {object} [properties.parametersLink] The URI referencing the
 * parameters. Use only one of Parameters or ParametersLink.
 * 
 * @member {string} [properties.parametersLink.uri] URI referencing the
 * template.
 * 
 * @member {string} [properties.parametersLink.contentVersion] If included it
 * must match the ContentVersion in the template.
 * 
 * @member {string} [properties.mode] The deployment mode. Possible values
 * include: 'Incremental', 'Complete'
 * 
 * @member {object} [properties.debugSetting] The debug setting of the
 * deployment.
 * 
 * @member {string} [properties.debugSetting.detailLevel] The debug detail
 * level.
 * 
 */
function DeploymentValidateResult() {
}

/**
 * Defines the metadata of DeploymentValidateResult
 *
 * @returns {object} metadata of DeploymentValidateResult
 *
 */
DeploymentValidateResult.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'DeploymentValidateResult',
    type: {
      name: 'Composite',
      className: 'DeploymentValidateResult',
      modelProperties: {
        error: {
          required: false,
          serializedName: 'error',
          type: {
            name: 'Composite',
            className: 'ResourceManagementErrorWithDetails'
          }
        },
        properties: {
          required: false,
          serializedName: 'properties',
          type: {
            name: 'Composite',
            className: 'DeploymentPropertiesExtended'
          }
        }
      }
    }
  };
};

module.exports = DeploymentValidateResult;
