/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

var util = require('util');

/**
 * @class
 * Initializes a new instance of the CloudPool class.
 * @constructor
 * A pool in the Azure Batch service.
 * @member {string} [id] A string that uniquely identifies the pool within the
 * account. The id can contain any combination of alphanumeric characters
 * including hyphens and underscores, and cannot contain more than 64
 * characters.
 * 
 * @member {string} [displayName] The display name for the pool.
 * 
 * @member {string} [url] The URL of the pool.
 * 
 * @member {string} [eTag] The ETag of the pool.
 * 
 * @member {date} [lastModified] The last modified time of the pool.
 * 
 * @member {date} [creationTime] The creation time of the pool.
 * 
 * @member {string} [state] The current state of the pool. Possible values
 * include: 'active', 'deleting', 'upgrading'
 * 
 * @member {date} [stateTransitionTime] The time at which the pool entered its
 * current state.
 * 
 * @member {string} [allocationState] Whether the pool is resizing. Possible
 * values include: 'steady', 'resizing', 'stopping'
 * 
 * @member {date} [allocationStateTransitionTime] The time at which the pool
 * entered its current allocation state.
 * 
 * @member {string} [vmSize] The size of virtual machines in the pool. All
 * virtual machines in a pool are the same size.
 * 
 * @member {object} [cloudServiceConfiguration] The cloud service
 * configuration for the pool. This property and VirtualMachineConfiguration
 * are mutually exclusive and one of the properties must be specified.
 * 
 * @member {string} [cloudServiceConfiguration.osFamily] The Azure Guest OS
 * family to be installed on the virtual machines in the pool.
 * 
 * @member {string} [cloudServiceConfiguration.targetOSVersion] The Azure
 * Guest OS version to be installed on the virtual machines in the pool. The
 * default value is * which specifies the latest operating system version for
 * the specified OS family.
 * 
 * @member {string} [cloudServiceConfiguration.currentOSVersion] The Azure
 * Guest OS Version currently installed on the virtual machines in the pool.
 * This may differ from TargetOSVersion if the pool state is Upgrading.
 * 
 * @member {object} [virtualMachineConfiguration] The virtual machine
 * configuration for the pool. This property and CloudServiceConfiguration
 * are mutually exclusive and one of the properties must be specified.
 * 
 * @member {object} [virtualMachineConfiguration.imageReference] A reference
 * to the Azure Virtual Machines Marketplace image to use.
 * 
 * @member {string} [virtualMachineConfiguration.imageReference.publisher] The
 * publisher of the Azure Virtual Machines Marketplace image. For example,
 * Canonical or MicrosoftWindowsServer.
 * 
 * @member {string} [virtualMachineConfiguration.imageReference.offer] The
 * offer type of the Azure Virtual Machines Marketplace image. For example,
 * UbuntuServer or WindowsServer.
 * 
 * @member {string} [virtualMachineConfiguration.imageReference.sku] The SKU
 * of the Azure Virtual Machines Marketplace image. For example, 14.04.0-LTS
 * or 2012-R2-Datacenter.
 * 
 * @member {string} [virtualMachineConfiguration.imageReference.version] The
 * version of the Azure Virtual Machines Marketplace image. A value of
 * 'latest' can be specified to select the latest version of an image. If
 * omitted, the default is 'latest'.
 * 
 * @member {string} [virtualMachineConfiguration.nodeAgentSKUId] The SKU of
 * Batch Node Agent to be provisioned on the compute node. The Batch node
 * agent is a program that runs on each node in the pool, and provides the
 * command-and-control interface between the node and the Batch service.
 * There are different implementations of the node agent, known as SKUs, for
 * different operating systems.
 * 
 * @member {object} [virtualMachineConfiguration.windowsConfiguration] Windows
 * operating system settings on the virtual machine. This property must not
 * be specified if the ImageReference property specifies a Linux OS image.
 * 
 * @member {boolean}
 * [virtualMachineConfiguration.windowsConfiguration.enableAutomaticUpdates]
 * Whether automatic updates are enabled on the virtual machine. If omitted,
 * the default value is true.
 * 
 * @member {moment.duration} [resizeTimeout] The timeout for allocation of
 * compute nodes to the pool. In a Get Pool operation, this is the timeout
 * for the most recent resize operation. The default value is 10 minutes.
 * 
 * @member {object} [resizeError] Details of any error encountered while
 * performing the last resize on the pool. This property is set only if an
 * error occurred during the last pool resize, and only when the pool
 * AllocationState is Steady.
 * 
 * @member {string} [resizeError.code] An identifier for the pool resize
 * error. Codes are invariant and are intended to be consumed
 * programmatically.
 * 
 * @member {string} [resizeError.message] A message describing the pool resize
 * error, intended to be suitable for display in a user interface.
 * 
 * @member {array} [resizeError.values] A list of additional error details
 * related to the pool resize error.
 * 
 * @member {number} [currentDedicated] The number of compute nodes currently
 * in the pool.
 * 
 * @member {number} [targetDedicated] The desired number of compute nodes in
 * the pool. This property must have the default value if EnableAutoScale is
 * true. It is required if EnableAutoScale is false.
 * 
 * @member {boolean} [enableAutoScale] Whether the pool size should
 * automatically adjust over time. If true, the AutoScaleFormula property
 * must be set. If false, the TargetDedicated property must be set.
 * 
 * @member {string} [autoScaleFormula] A formula for the desired number of
 * compute nodes in the pool.
 * 
 * @member {moment.duration} [autoScaleEvaluationInterval] A time interval for
 * the desired AutoScale evaluation period in the pool.
 * 
 * @member {object} [autoScaleRun] The results and errors from the last
 * execution of the autoscale formula.
 * 
 * @member {date} [autoScaleRun.timestamp] The time at which the autoscale
 * formula was last evaluated.
 * 
 * @member {string} [autoScaleRun.results] The final values of all variables
 * used in the evaluation of the autoscale formula. Each variable value is
 * returned in the form $variable=value, and variables are separated by
 * semicolons.
 * 
 * @member {object} [autoScaleRun.error] Details of the error encountered
 * evaluating the autoscale formula on the pool, if the evaluation was
 * unsuccessful.
 * 
 * @member {string} [autoScaleRun.error.code] An identifier for the autoscale
 * error. Codes are invariant and are intended to be consumed
 * programmatically.
 * 
 * @member {string} [autoScaleRun.error.message] A message describing the
 * autoscale error, intended to be suitable for display in a user interface.
 * 
 * @member {array} [autoScaleRun.error.values] A list of additional error
 * details related to the autoscale error.
 * 
 * @member {boolean} [enableInterNodeCommunication] Whether the pool permits
 * direct communication between nodes.
 * 
 * @member {object} [startTask] A task specified to run on each compute node
 * as it joins the pool.
 * 
 * @member {string} [startTask.commandLine] The command line of the start task.
 * 
 * @member {array} [startTask.resourceFiles] A list of files that the Batch
 * service will download to the compute node before running the command line.
 * 
 * @member {array} [startTask.environmentSettings] A list of environment
 * variable settings for the start task.
 * 
 * @member {boolean} [startTask.runElevated] Whether to run the start task in
 * elevated mode. The default value is false.
 * 
 * @member {number} [startTask.maxTaskRetryCount] The maximum number of times
 * the task may be retried.
 * 
 * @member {boolean} [startTask.waitForSuccess] Whether the Batch service
 * should wait for the start task to complete successfully (that is, to exit
 * with exit code 0) before scheduling any tasks on the compute node.
 * 
 * @member {array} [certificateReferences] The list of certificates to be
 * installed on each compute node in the pool.
 * 
 * @member {array} [applicationPackageReferences] The list of application
 * packages to be installed on each compute node in the pool.
 * 
 * @member {number} [maxTasksPerNode] The maximum number of tasks that can run
 * concurrently on a single compute node in the pool.
 * 
 * @member {object} [taskSchedulingPolicy] How the Batch service distributes
 * tasks between compute nodes in the pool.
 * 
 * @member {string} [taskSchedulingPolicy.nodeFillType] How tasks should be
 * distributed across compute nodes. Possible values include: 'spread',
 * 'pack', 'unmapped'
 * 
 * @member {array} [metadata] A list of name-value pairs associated with the
 * pool as metadata.
 * 
 * @member {object} [stats] Utilization and resource usage statistics for the
 * entire lifetime of the pool.
 * 
 * @member {string} [stats.url] The URL for the statistics.
 * 
 * @member {date} [stats.startTime] The start time of the time range covered
 * by the statistics.
 * 
 * @member {date} [stats.lastUpdateTime] The time at which the statistics were
 * last updated. All statistics are limited to the range between startTime
 * and lastUpdateTime.
 * 
 * @member {object} [stats.usageStats] Statistics related to pool usage, such
 * as the amount of core-time used.
 * 
 * @member {date} [stats.usageStats.startTime] The start time of the time
 * range covered by the statistics.
 * 
 * @member {date} [stats.usageStats.lastUpdateTime] The time at which the
 * statistics were last updated. All statistics are limited to the range
 * between startTime and lastUpdateTime.
 * 
 * @member {moment.duration} [stats.usageStats.dedicatedCoreTime] The
 * aggregated wall-clock time of the dedicated compute node cores being part
 * of the pool.
 * 
 * @member {object} [stats.resourceStats] Statistics related to resource
 * consumption by compute nodes in the pool.
 * 
 * @member {date} [stats.resourceStats.startTime] The start time of the time
 * range covered by the statistics.
 * 
 * @member {date} [stats.resourceStats.lastUpdateTime] The time at which the
 * statistics were last updated. All statistics are limited to the range
 * between startTime and lastUpdateTime.
 * 
 * @member {number} [stats.resourceStats.avgCPUPercentage] The average CPU
 * usage across all nodes in the pool (percentage per node).
 * 
 * @member {number} [stats.resourceStats.avgMemoryGiB] The average memory
 * usage in GiB across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.peakMemoryGiB] The peak memory usage
 * in GiB across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.avgDiskGiB] The average used disk
 * space in GiB across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.peakDiskGiB] The peak used disk space
 * in GiB across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.diskReadIOps] The total number of
 * disk read operations across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.diskWriteIOps] The total number of
 * disk write operations across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.diskReadGiB] The total amount of data
 * in GiB of disk reads across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.diskWriteGiB] The total amount of
 * data in GiB of disk writes across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.networkReadGiB] The total amount of
 * data in GiB of network reads across all nodes in the pool.
 * 
 * @member {number} [stats.resourceStats.networkWriteGiB] The total amount of
 * data in GiB of network writes across all nodes in the pool.
 * 
 */
function CloudPool() {
}

/**
 * Defines the metadata of CloudPool
 *
 * @returns {object} metadata of CloudPool
 *
 */
CloudPool.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'CloudPool',
    type: {
      name: 'Composite',
      className: 'CloudPool',
      modelProperties: {
        id: {
          required: false,
          serializedName: 'id',
          type: {
            name: 'String'
          }
        },
        displayName: {
          required: false,
          serializedName: 'displayName',
          type: {
            name: 'String'
          }
        },
        url: {
          required: false,
          serializedName: 'url',
          type: {
            name: 'String'
          }
        },
        eTag: {
          required: false,
          serializedName: 'eTag',
          type: {
            name: 'String'
          }
        },
        lastModified: {
          required: false,
          serializedName: 'lastModified',
          type: {
            name: 'DateTime'
          }
        },
        creationTime: {
          required: false,
          serializedName: 'creationTime',
          type: {
            name: 'DateTime'
          }
        },
        state: {
          required: false,
          serializedName: 'state',
          type: {
            name: 'Enum',
            allowedValues: [ 'active', 'deleting', 'upgrading' ]
          }
        },
        stateTransitionTime: {
          required: false,
          serializedName: 'stateTransitionTime',
          type: {
            name: 'DateTime'
          }
        },
        allocationState: {
          required: false,
          serializedName: 'allocationState',
          type: {
            name: 'Enum',
            allowedValues: [ 'steady', 'resizing', 'stopping' ]
          }
        },
        allocationStateTransitionTime: {
          required: false,
          serializedName: 'allocationStateTransitionTime',
          type: {
            name: 'DateTime'
          }
        },
        vmSize: {
          required: false,
          serializedName: 'vmSize',
          type: {
            name: 'String'
          }
        },
        cloudServiceConfiguration: {
          required: false,
          serializedName: 'cloudServiceConfiguration',
          type: {
            name: 'Composite',
            className: 'CloudServiceConfiguration'
          }
        },
        virtualMachineConfiguration: {
          required: false,
          serializedName: 'virtualMachineConfiguration',
          type: {
            name: 'Composite',
            className: 'VirtualMachineConfiguration'
          }
        },
        resizeTimeout: {
          required: false,
          serializedName: 'resizeTimeout',
          type: {
            name: 'TimeSpan'
          }
        },
        resizeError: {
          required: false,
          serializedName: 'resizeError',
          type: {
            name: 'Composite',
            className: 'ResizeError'
          }
        },
        currentDedicated: {
          required: false,
          serializedName: 'currentDedicated',
          type: {
            name: 'Number'
          }
        },
        targetDedicated: {
          required: false,
          serializedName: 'targetDedicated',
          type: {
            name: 'Number'
          }
        },
        enableAutoScale: {
          required: false,
          serializedName: 'enableAutoScale',
          type: {
            name: 'Boolean'
          }
        },
        autoScaleFormula: {
          required: false,
          serializedName: 'autoScaleFormula',
          type: {
            name: 'String'
          }
        },
        autoScaleEvaluationInterval: {
          required: false,
          serializedName: 'autoScaleEvaluationInterval',
          type: {
            name: 'TimeSpan'
          }
        },
        autoScaleRun: {
          required: false,
          serializedName: 'autoScaleRun',
          type: {
            name: 'Composite',
            className: 'AutoScaleRun'
          }
        },
        enableInterNodeCommunication: {
          required: false,
          serializedName: 'enableInterNodeCommunication',
          type: {
            name: 'Boolean'
          }
        },
        startTask: {
          required: false,
          serializedName: 'startTask',
          type: {
            name: 'Composite',
            className: 'StartTask'
          }
        },
        certificateReferences: {
          required: false,
          serializedName: 'certificateReferences',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'CertificateReferenceElementType',
                type: {
                  name: 'Composite',
                  className: 'CertificateReference'
                }
            }
          }
        },
        applicationPackageReferences: {
          required: false,
          serializedName: 'applicationPackageReferences',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'ApplicationPackageReferenceElementType',
                type: {
                  name: 'Composite',
                  className: 'ApplicationPackageReference'
                }
            }
          }
        },
        maxTasksPerNode: {
          required: false,
          serializedName: 'maxTasksPerNode',
          type: {
            name: 'Number'
          }
        },
        taskSchedulingPolicy: {
          required: false,
          serializedName: 'taskSchedulingPolicy',
          type: {
            name: 'Composite',
            className: 'TaskSchedulingPolicy'
          }
        },
        metadata: {
          required: false,
          serializedName: 'metadata',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'MetadataItemElementType',
                type: {
                  name: 'Composite',
                  className: 'MetadataItem'
                }
            }
          }
        },
        stats: {
          required: false,
          serializedName: 'stats',
          type: {
            name: 'Composite',
            className: 'PoolStatistics'
          }
        }
      }
    }
  };
};

module.exports = CloudPool;
