/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

/**
 * @class
 * Initializes a new instance of the DeploymentValidateResult class.
 * @constructor
 * Information from validate template deployment response.
 * @member {object} [error] Gets or sets validation error.
 * 
 * @member {string} [error.code] Gets or sets the error code returned from the
 * server.
 * 
 * @member {string} [error.message] Gets or sets the error message returned
 * from the server.
 * 
 * @member {string} [error.target] Gets or sets the target of the error.
 * 
 * @member {array} [error.details] Gets or sets validation error.
 * 
 * @member {object} [properties] Gets or sets the template deployment
 * properties.
 * 
 * @member {string} [properties.provisioningState] Gets or sets the state of
 * the provisioning.
 * 
 * @member {string} [properties.correlationId] Gets or sets the correlation ID
 * of the deployment.
 * 
 * @member {date} [properties.timestamp] Gets or sets the timestamp of the
 * template deployment.
 * 
 * @member {object} [properties.outputs] Gets or sets key/value pairs that
 * represent deploymentoutput.
 * 
 * @member {array} [properties.providers] Gets the list of resource providers
 * needed for the deployment.
 * 
 * @member {array} [properties.dependencies] Gets the list of deployment
 * dependencies.
 * 
 * @member {object} [properties.template] Gets or sets the template content.
 * Use only one of Template or TemplateLink.
 * 
 * @member {object} [properties.templateLink] Gets or sets the URI referencing
 * the template. Use only one of Template or TemplateLink.
 * 
 * @member {string} [properties.templateLink.uri] URI referencing the template.
 * 
 * @member {string} [properties.templateLink.contentVersion] If included it
 * must match the ContentVersion in the template.
 * 
 * @member {object} [properties.parameters] Deployment parameters. Use only
 * one of Parameters or ParametersLink.
 * 
 * @member {object} [properties.parametersLink] Gets or sets the URI
 * referencing the parameters. Use only one of Parameters or ParametersLink.
 * 
 * @member {string} [properties.parametersLink.uri] URI referencing the
 * template.
 * 
 * @member {string} [properties.parametersLink.contentVersion] If included it
 * must match the ContentVersion in the template.
 * 
 * @member {string} [properties.mode] Gets or sets the deployment mode.
 * Possible values include: 'Incremental', 'Complete'
 * 
 * @member {object} [properties.debugSetting] Gets or sets the debug setting
 * of the deployment.
 * 
 * @member {string} [properties.debugSetting.detailLevel] Gets or sets the
 * debug detail level.
 * 
 */
function DeploymentValidateResult() {
}

/**
 * Defines the metadata of DeploymentValidateResult
 *
 * @returns {object} metadata of DeploymentValidateResult
 *
 */
DeploymentValidateResult.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'DeploymentValidateResult',
    type: {
      name: 'Composite',
      className: 'DeploymentValidateResult',
      modelProperties: {
        error: {
          required: false,
          serializedName: 'error',
          type: {
            name: 'Composite',
            className: 'ResourceManagementErrorWithDetails'
          }
        },
        properties: {
          required: false,
          serializedName: 'properties',
          type: {
            name: 'Composite',
            className: 'DeploymentPropertiesExtended'
          }
        }
      }
    }
  };
};

module.exports = DeploymentValidateResult;
